function DOKarma:DObyte64(byte)
	if (byte == nil) then result = "="
	elseif (byte <= 25) then result = string.char(65+byte)
	elseif (byte <= 51) then result = string.char(71+byte)
	elseif (byte <= 61) then result = string.char(byte-4)
	elseif (byte == 62) then result = "-"
	elseif (byte == 63) then result = "_"
	else result = nil end
	return result
end

-- function encodeblock
-- encodes a 3 bytes long input string block to 4 base64 characters.
function DOKarma:DOencodeblock(block)
	local result = ""
	local bytes = {}
	local sixbits = {}
	for byte=1,3 do bytes[byte] = string.byte(string.sub(block,byte)) or 0 end
	sixbits[1] = self:DObyte64(math.floor(bytes[1]/4))
	sixbits[2] = self:DObyte64((bytes[1] % 4) * 16 + math.floor(bytes[2] / 16))
	if (string.len(block) > 1) then
		sixbits[3] = self:DObyte64((bytes[2] % 16) * 4 + math.floor(bytes[3] / 64))
	else
		sixbits[3] = "="
	end
	if (string.len(block) > 2) then
		sixbits[4] = self:DObyte64(bytes[3] % 64)
	else
		sixbits[4] = "="
	end
	for nr,sixbit in pairs(sixbits) do
		result = result .. sixbit
	end 
	return result
end

-- function encode
-- encodes input string to base64
function DOKarma:DOencode(input)
	local result = ""
	for c=1,string.len(input),3 do
		result = result .. self:DOencodeblock(string.sub(input,c,c+2))
	end
	return result
end

-- function 6bit
-- returns the 6bit value of a base64 character
function DOKarma:DOsixbit(char)
	local byte = string.byte(char)
	local result = nil
	if (byte == 61) then result = 0
	elseif (byte == 45 or byte == 43) then result = 62
	elseif (byte == 95 or byte == 47) then result = 63
	elseif (byte <= 57) then result = byte + 4
	elseif (byte <= 90) then result = byte - 65
	elseif (byte <= 122) then result = byte - 71
	end
	return result
end

-- function decodeblock
-- decodes a 4 byte base64 block to a 3 byte string block
function DOKarma:DOdecodeblock(block)
	local sixbits = {}
	local result = ""
	for counter=1,4 do sixbits[counter] = self:DOsixbit(string.sub(block,counter,counter)) end
	result = string.char(sixbits[1]*4 + math.floor(sixbits[2] / 16))
	if (string.sub(block,3,3) ~= "=") then
		result = result .. string.char((sixbits[2] % 16)*16 + math.floor(sixbits[3] / 4))
	end
	if (string.sub(block,4,4) ~= "=") then
		result = result .. string.char((sixbits[3] % 4) * 64 + sixbits[4])
	end
	return result
end

-- function decode
-- Decodes base64-encoded data
function DOKarma:DOdecode(data)
	local result = ""
	for c=1,string.len(data),4 do
		result = result .. self:DOdecodeblock(string.sub(data,c,c+3))
	end
	return result
end

function DOKarma:TestFunction()
end


----------------------------------------------
-- Pickle.lua
-- A table serialization utility for lua
-- Steve Dekorte, http://www.dekorte.com, Apr 2000
-- Freeware
----------------------------------------------

function DOKarma:pickle(t)
  return self:pickle_(DOKarma:PickleClone(t))
end

function DOKarma:PickleClone(t)
	local nt={}
	for i, v in pairs(t) do
		nt[i]=v
	end
	return nt
end

function DOKarma:pickle_(root)
  if type(root) ~= "table" then 
    error("can only pickle tables, not ".. type(root).."s")
  end
  self._tableToRef = {}
  self._refToTable = {}
  local savecount = 0
  self:ref_(root)
  local s = ""

  while table.getn(self._refToTable) > savecount do
    savecount = savecount + 1
    local t = self._refToTable[savecount]
    s = s.."{"
    for i, v in pairs(t) do
        s = string.format("%s[%s]=%s,", s, self:value_(i), self:value_(v))
    end
    s = s.."},"
  end

  return string.format("{%s}", s)
end

function DOKarma:value_(v)
  local vtype = type(v)
  if     vtype == "string" then return string.format("%q", v)
  elseif vtype == "number" then return v
  elseif vtype == "table" then return "{"..self:ref_(v).."}"
  else --error("pickle a "..type(v).." is not supported")
  end  
end

function DOKarma:ref_(t)
  local ref = self._tableToRef[t]
  if not ref then 
    if t == self then error("can't pickle the pickle class") end
    table.insert(self._refToTable, t)
    ref = table.getn(self._refToTable)
    self._tableToRef[t] = ref
  end
  return ref
end

----------------------------------------------
-- unpickle
----------------------------------------------

function DOKarma:unpickle(s)
  if type(s) ~= "string" then
    error("can't unpickle a "..type(s)..", only strings")
  end
  local gentables = loadstring("return "..s)
  local tables = gentables()
  
  for tnum = 1, table.getn(tables) do
    local t = tables[tnum]
    local tcopy = {}; for i, v in pairs(t) do tcopy[i] = v end
    for i, v in pairs(tcopy) do
      local ni, nv
      if type(i) == "table" then ni = tables[i[1]] else ni = i end
      if type(v) == "table" then nv = tables[v[1]] else nv = v end
      t[ni] = nv
    end
  end
  return tables[1]
end

